@@ -57,7 +57,10 @@ FetCamImage::FetCamImage(
     messenger(0),
     currentReply(0),
     timer(0),
-    numberOfRedirections(0)
+    currentTimerInterval(5000),
+    numberOfRedirections(0),
+    networkAccessible(true),
+    sleeping(false)
 {
   setAlignment(Qt::AlignCenter);
 
@@ -116,6 +119,16 @@ FetCamImage::FetCamImage(
     this,
     SLOT(updateWidget(QNetworkReply *)));
 
+  connect(
+    &qnam,
+    SIGNAL(
+      networkAccessibleChanged(
+        QNetworkAccessManager::NetworkAccessibility)),
+    this,
+    SLOT(
+      parseNetworkAccessibility(
+        QNetworkAccessManager::NetworkAccessibility)));
+
   connect(
     timer,
     SIGNAL(timeout()),
@@ -136,6 +149,22 @@ FetCamImage::FetCamImage(
     // Start with the first item on the list:
     selectorDialog->chooseItem(0);
   }
+
+#ifdef MAEMO_OS
+  // Turn off Fettuccine when the display goes dark:
+  connect(
+    &dbusListener,
+    SIGNAL(timeToSleep()),
+    this,
+    SLOT(enterSleepMode()));
+
+  // Turn back on when the screen comes back up:
+  connect(
+    &dbusListener,
+    SIGNAL(timeToWakeUp()),
+    this,
+    SLOT(exitSleepMode()));
+#endif // MAEMO_OS
 }
 
 
@@ -345,10 +374,11 @@ void FetCamImage::loadWebcam(
   currentWebcamUrl = item->getLink();
   currentWebcamHomepage = item->getHomepage();
   emit newWebcamName(item->getName());
+  currentTimerInterval = item->getRefreshRate() * 1000;
 
   retrieveImage();
 
-  timer->start(item->getRefreshRate() * 1000);
+  timer->start(currentTimerInterval);
 }
 
 
@@ -382,7 +412,7 @@ void FetCamImage::setImage(
   // Sanity check:
   if (!ir.canRead())
   {
-    emit imageError("Unable to display webcam picture");
+    messenger->sendMessage("Unable to display webcam picture");
     return;
   }
 
@@ -456,3 +486,77 @@ void FetCamImage::resizeEvent(
 
   event->accept();
 }
+
+
+void FetCamImage::parseNetworkAccessibility(
+  QNetworkAccessManager::NetworkAccessibility access)
+{
+  if (access == QNetworkAccessManager::Accessible)
+  {
+    // If the network was previously inaccessible, restart:
+    if (!networkAccessible)
+    {
+      networkAccessible = true;
+
+      // But only restart if we are not sleeping:
+      if (!sleeping)
+      {
+        timer->start(currentTimerInterval);
+      }
+    }
+  }
+  else
+  {
+    // If the network was previously accessible, stop:
+    if (networkAccessible)
+    {
+      networkAccessible = false;
+
+      timer->stop();
+
+      if (currentReply)
+      {
+        currentReply->abort();
+        currentReply->deleteLater();
+        currentReply = 0;
+      }
+    }
+  }
+}
+
+
+void FetCamImage::enterSleepMode()
+{
+  // Shut down the timer, and abort any current requests:
+  sleeping = true;
+
+  timer->stop();
+
+  if (currentReply)
+  {
+    currentReply->abort();
+    currentReply->deleteLater();
+    currentReply = 0;
+  }
+}
+
+
+void FetCamImage::exitSleepMode()
+{
+  sleeping = false;
+
+  // First check if the network is accessible:
+  if (networkAccessible)
+  {
+    // Refresh the image, and start the timer back up again.
+    retrieveImage();
+    timer->start(currentTimerInterval);
+  }
+  // The following is a hack to get around a QNAM problem:
+  else if (qnam.networkAccessible() ==
+    QNetworkAccessManager::UnknownAccessibility)
+  {
+    // Force the QNAM to wake up:
+    retrieveImage();
+  }
+}
